/**
　Dashboard widgetでイベントによってボックスの大きさをアニメーションで変更するためのクラス．
  以下のようにして使用する．
 カウントアップするごとにイベントの通知を受け取りたい場合はaddListener/removeListenerを使用すること．

<script>
 var obj = new Expander( document.getElementById( 'target' ), endHandler, errorHandler );
 obj.setSize( width0, height0, width1, height1 );
//obj.setInterval( [animation speed] );
//obj.setSteps( [animation steps] );
//obj.addListener( listerfunction );
</script>

 <a onClick='obj.expand(event);'>Click</a>

*/
// ログ出力
function __LOG( message ) {
    var status = document.getElementById( 'status' );
    if ( status ) {
        status.innerText = message;
    }
}

// コンストラクタ．サンプルコードを多いに参考にさせてもらった．
function Expander( mainElement, endHandler, errorHandler ) {
    this.element      = mainElement;
    if ( endHandler == undefined ) endHandler = null;//function(){}
    this.endHandler   = endHandler;
    if ( errorHandler == undefined ) errorHandler = __LOG;
    this.errorHandler = errorHandler;

    // rectangle
    this.width = 0;
    this.height = 0;
    this.width_1     = 200;
    this.height_1    = 400;
    this.width_2     = 400;
    this.height_2    = 400;

    // motion control
    this.steps       = 10;
    this.counter     = 0;
    this.timer       = null;
    this.isRunning   = false;
    this.isCollapsed = false;
    this.interval    = 13;

    // event listeners
    this.eventListeners = new Array();

    this.modeX = true;
    this.modeY = false;

    // private methods, do not call from HTML
    this.__tick           = _Expander_tick;
    this.__getXOffset     = _Expander_getXOffset;
    this.__getYOffset     = _Expander_getYOffset;
    this.__setOffsetFlags = _Expander_setOffsetFlags;
    // public methods
    this.setSize          = _Expander_setSize;
    this.setSteps         = _Expander_setSteps;
    this.expand           = _Expander_expand;
    this.setInterval      = _Expander_setInterval;
    this.addListener      = _Expander_addListener;
    this.removeListener   = _Expander_removeListener;

    // initialize
    this.__setOffsetFlags();
}

// イベントリスナーの追加．リスナーは関数で，Expanderオブジェクトが
//引数としてそのまま渡される
function _Expander_addListener( listener ) {
    this.eventListeners.push( listener );
}

// イベントリスナーの除去
function _Expander_removeListener( listener ) {
    for ( var i = 0; i < this.eventListeners.length; i++ ) {
        if ( this.eventListeners[ i ] == listener ) {
            this.eventListeners.splice( i, 1 );
            break;
        }
    }
}

//サイズ変更する初期値と変化値を決める
function _Expander_setSize( w1, h1, w2, h2 ) {
    this.width_1  = w1;
    this.height_1 = h1;
    this.width_2  = w2;
    this.height_2 = h2;
    this.__setOffsetFlags();
}

// アニメーションの速度を設定する．短いほど速い
function _Expander_setInterval( interval ) {
    this.interval = interval;
}

// 移動の細かさを設定する．大きいほど細かい
function _Expander_setSteps( steps ) {
    if ( this.isRunning ) {
        return;
    }
    this.steps = steps;
}

// 移動の実行
function _Expander_expand( event ) {
    if ( this.isRunning ) {
        return;
        //this.errorHandler( "cannot move while running" );
    }
//     if ( this.element ) {
//         this.errorHandler( "No element" );
//     }
    if ( this.element != null ) {
        this.isRunning = true;
        var self = this;
        this.counter = 0;
        this.timer = setInterval( function(){ self.__tick(); }, this.interval );
        if ( this.isCollapsed ) {
            this.width  = this.width_2;
            this.height = this.height_2;
        } else {
            this.width  = this.width_1;
            this.height = this.height_1;
        }
        this.__tick();
    } else {
        this.errorHandler( "cannot start" );
    }
}

function _Expander_setOffsetFlags() {
    this.modeX = ( this.width_1 != this.width_2 );
    this.modeY = ( this.height_1 != this.height_2 );
}

// X方向の移動距離を求める
function _Expander_getXOffset() {
    if ( this.modeX == false ) return 0;
    var t1 = ( this.counter - 1 )/ this.steps * .5 * Math.PI;
    var t2 = ( this.counter ) / this.steps * .5 * Math.PI;
    var offsetX = ( Math.sin( t2 ) - Math.sin( t1 ) ) * ( this.width_2 - this.width_1 );
    if ( this.isCollapsed ) {
        offsetX = -offsetX;
    }
    return offsetX;
}

// Y方向の移動距離を求める
function _Expander_getYOffset() {
    if ( this.modeY == false ) return 0;
    var t1 = ( this.counter - 1 )/ this.steps * .5 * Math.PI;
    var t2 = ( this.counter ) / this.steps * .5 * Math.PI;
    var offsetY = ( Math.sin( t2 ) - Math.sin( t1 ) ) * ( this.height_2 - this.height_1 );
    if ( this.isCollapsed ) {
        offsetY = -offsetY;
    }
    return offsetY;
}

// 移動の実行．一定の時間が経過していた場合アニメーションを止める
function _Expander_tick() {
    this.counter ++;
    for ( var i = 0; i < this.eventListeners.length; i++ ) {
        this.eventListeners[ i ]( this );
    }
    if ( this.counter >= this.steps ) {
        this.isRunning = false;
        if ( this.isCollapsed ) {
            var offsetX = parseInt( this.width_1 - this.width );
            var offsetY = parseInt( this.heght_1 - this.height );
            window.resizeBy( offsetX, offsetY );
            this.width  = this.width_1;
            this.height = this.height_1;
            this.element.style.width  = this.width   + "px";
            this.element.style.height = this.height  + "px";
            this.isCollapsed = false;
        } else {
            var offsetX = parseInt( this.width_2 - this.width );
            var offsetY = parseInt( this.height_2 - this.height );
            window.resizeBy( offsetX, offsetY );
            this.width  = this.width_2;
            this.height = this.height_2;
            this.element.style.width  = this.width  + "px";
            this.element.style.height = this.height + "px";
            this.isCollapsed = true;
        }
        if ( this.timer != null ) {
            clearInterval( this.timer );
            timer = null;
        }
        if ( this.endHandler != null ) {
            //if ( this.endHandler != undefined && this.endHandler == null ) {
            this.endHandler( this );
        }
    } else {
        var offsetX = parseInt( this.__getXOffset() );
        var offsetY = parseInt( this.__getYOffset() );
        window.resizeBy( offsetX, offsetY );
        this.width  += offsetX;
        this.height += offsetY;
        this.element.style.width  = this.width  + "px";
        this.element.style.height = this.height + "px";
    }
}
